/*
 *		CBA header file
 *
 */

/*
 *	Detect operating system
 */

#if defined(linux)
# define OS_Linux
#elif defined(sun)
# if defined(__SVR4)
#  define OS_Solaris
# else
#  define OS_SunOS
# endif
#elif defined(__FreeBSD__) || defined(__NetBSD__)
# define OS_BSD
#elif defined(__SVR4)
# define OS_SVR4
#else
	Error : Unknown Operating System.
#endif

#include <stdio.h>
#include <sys/time.h>	/* for gettimeofday(2)	*/
#include <time.h>	/* for localtime(3)	*/
#include <string.h>	/* for strcmp(3)	*/
#include <stdlib.h>	/* for ttyname(3), lockf(3), atoi(3)	*/
#include <unistd.h>	/* for ttyname(3), lockf(3)		*/
#include <termios.h>	/* for tcsetattr(3), TIOCM_XXX		*/
#include <signal.h>	/* for signal(2)	*/
#include <sys/wait.h>	/* for wait(2)		*/
#include <sys/stat.h>	/* for stat(2)		*/
#if defined(OS_Solaris)
# include <utmpx.h>	/* for utmp(2)		*/
#else
# include <utmp.h>	/* for utmp(2)		*/
#endif
#include <netdb.h>	/* for gethostbyname(2)	*/
#include <netinet/in.h>	/* for inet_ntoa(3)	*/
#include <arpa/inet.h>	/* for inet_ntoa(3)	*/
#include <fcntl.h>	/* for open(2)		*/
#include <pwd.h>	/* for getpwnam(2)	*/
#if !defined(OS_SunOS)
#include <sys/ioctl.h>	/* for ioctl(2)		*/
#endif

#if !defined(OS_Solaris) && !defined(OS_SVR4)
# include <sys/file.h>	/* for flock(3)		*/
#endif

#ifdef DIRECT
# if defined(OS_Solaris)
#  include <crypt.h>	/* for crypt(3)		*/
# endif
#endif

#include <errno.h>

#if defined(OS_SunOS)
/* SunOS 4.1.2 JLE has no sys_errlist[].	*/
/* if your SunOS has sys_errlist[],		*/
/* use the following definition.		*/
/* */
/* # define strerror(X) sys_errlist[X]		*/
/* */
/* Otherwise, use the following	macros		*/
# define strerror(X) cba_strerror(X)	/*	*/
# define NO_SYS_ERRLIST			/*	*/
/* */
#endif

/*
 *	System constants
 */
#define MAXPHONES	512	/* max.# of phone numbers		*/
#define MAXMTYPES	64	/* max.# of modem config. types		*/
#define MAXFLINE	320	/* max.# + 1 of a line of config. files	*/
#define MAXFPATH	80	/* max.# + 1 of a setup file path	*/

/* for pho file */
#define CALLIN		"!callin"	/* tel. # authentication */
#define DIRECT		"!direct"	/* passwd authentication */
#define PPPCALLIN	"!ppp"		/* PPP callin */


/* for user struct */
#define MAXNAME		16	/* max.# + 1 of login name 		*/
#define MAXPHONE	16	/* max.# + 1 of phone number		*/
#define MAXPHONEID	16	/* max.# + 1 of phone ID		*/
#define MAXSETUP	128	/* max.# + 1 of setup string		*/
#define MAXDIAL		64	/* max.# + 1 of dialing string		*/
#define MAXTRACE	16	/* max.# + 1 of modem trace comm.	*/
#define MAXAKEY		16	/* max.# + 1 of application key		*/
#define MAXTERM		16	/* max. len. + 1 of terminal string 	*/
#define MAXAARGV	32	/* max.# of application argv		*/

#define TERM_ROW	24	/* default terminal size in char.	*/
#define TERM_COL	80

#if defined(UT_HOSTSIZE)
# define MAXPNAME	(UT_HOSTSIZE + 1)
#else
# define MAXPNAME	32	/* max.# + 1 of phone name 		*/
#endif

/*
 *	Locations of accounting files
 */

#define UTMPFILE	"/tmp/cba.utmp"

#if defined(_PATH_UTMP)		/* for FreeBSD */
/* */
# define SYSUTMP	_PATH_UTMP
# define SYSWTMP	_PATH_WTMP
/* */
#elif defined(UTMPX_FILE)	/* for Solaris */
/* */
# define SYSUTMP	UTMPX_FILE
# define SYSWTMP	WTMPX_FILE
/* */
#elif defined(UTMP_FILE)	/* for Linux */
/* */
# define SYSUTMP	UTMP_FILE
# define SYSWTMP	WTMP_FILE
/* */
#elif defined(OS_SunOS)		/* for SunOS */
/* */
# define SYSUTMP	"/etc/utmp"
# define SYSWTMP	"/var/adm/wtmp"
/* */
#else				/* for others */
/* */
# define SYSUTMP	"xxxx/utmp"
# define SYSWTMP	"xxxx/wtmp"
/* */
#endif


/*
 *	Other unix programs
 */
#ifdef OS_SunOS
# define LAST	"/usr/ucb/last"
#else
# define LAST	"/usr/bin/last"
#endif

/*
 *	Structures
 *
 */

struct sys
{
    char *verify;		/* access verification string	*/
    int	 to_ac, to_in, to_back;	/* timeouts		*/
    int	 intvl;			/* callback interval	*/
    int	 retry;			/* retry count		*/
    int	 maxuse;		/* use hours time limit	*/
    int	 monint;		/* monitor interval	*/
    int	 maxwait;		/* callin waiting time limit	*/
    char *nserv1, *nserv2;	/* microsoft specific name server */
};

struct mdm
{
    char *name;			/* modem name		*/

    char *setup;		/* modem setup string	*/
    char *dial;			/* modem dial string	*/
    char *hook;			/* modem onhook		*/
    char *reset;		/* modem reset string	*/
    char *hsync;		/* modem sync. string after break */
    int  hang_method;		/* line hangup method.	*/
					/* 0 by hangup string */
					/* HUP_BREAK by break */
					/* HUP_DTR   by DTR(ER) */
    char *hang;			/* line hangup string	*/
    char *esc;			/* response to escape mode */
    struct
    {
	int type;			/* 0: empty, 1: startup, -1: end */
	char *modem;			/* trace command	*/
	char *name;			/* print name		*/
    }	trace[MAXTRACE];	/* trace modem command */
};

#define HUP_EMPTY	0
#define HUP_BREAK	0x01
#define HUP_DTR		0x02

#define TT_EMPTY	0
#define TT_INIT		0x01
#define TT_RING		0x02
#define TT_BACK		0x04
#define TT_RTRY		0x08
#define TT_DISC		0x80


struct dev
{
    char *name;			/* tty name		*/
    struct termios tio;		/* terminal attribute	*/
    char *baud;			/* baud rate 		*/

    struct mdm modem;		/* modem info.		*/
    char *dext;			/* tel. no. ext.	*/
    char *ip;			/* my IP address	*/
    char *peer;			/* peer IP address	*/
};

struct mtyp
{
    char name[MAXNAME];		/* modem configuration name	*/
    char setup[MAXSETUP];	/* modem setup string		*/
    char dial[MAXDIAL];		/* modem dial string if exists	*/
};

struct user
{
    char name[MAXNAME];		/* login name 		*/
    char phone[MAXPHONE];	/* phone number		*/
    char pid[MAXPHONEID];	/* phone ID		*/
    char where[MAXPNAME];	/* printed name of the place */
    char setup[MAXSETUP];	/* modem setup string	*/
    char appli[MAXAKEY];	/* application key	*/
    char term[MAXTERM];		/* terminal		*/
};

struct vars
{
    char name;			/* name			*/
    char *val;			/* value		*/
};

enum errclass { SECUR, SYSTM, MAINT, TRACE };

#define TRACE_CTRL	1
#define TRACE_MODEM	2
#define TRACE_TTY	4
#define TRACE_ULOG	8
#define TRACE_FILE	16

#define NUL '\000'

/*
 *	System dependent routines
 */

#if defined(OS_SVR4) || defined(OS_Solaris)
# define LOCKFILE(fd)	lockf(fd, F_LOCK, 0)
# define UNLKFILE(fd)	lockf(fd, F_ULOCK, 0)
#else
# define LOCKFILE(fd)	flock(fd, LOCK_EX)
# define UNLKFILE(fd)	flock(fd, LOCK_UN)
#endif


/*
 *	wait t/10 sec.
 */

#ifdef NO_USLEEP
extern lsleep();
#else
# define lsleep(t)	usleep(t * 100000)
#endif

